ScriptName = Lil' Builder
RegExMatch("$LastChangedRevision: 18 $"
         . "$LastChangedDate: 2007-06-06 14:00:35 +0200 (Mi, 06 Jun 2007) $"
         , "(?P<Num>\d+).*?(?P<Date>(?:\d+-?){3})", SVN_Rev)
CheckRequiredAHKVersion("1.0.46.17")

/*
www.autohotkey.com/forum/topic19484.html
Original by Majkinetor
modifications by: Rajat, Ahklerner, Majkinetor, Toralf

If you want to help, post your changes in the forum thread. Thanks.
*/

#NoEnv
#SingleInstance force
SetWorkingDir, %A_ScriptDir%
SetBatchLines, -1
SetControlDelay, -1 
CoordMode, Mouse, Screen 
OnExit, LilBuilder_OnExit
Dic("initialize")

LilBuilder()

; ListVars       ;debug: check for global vars

Return
;####################  End  of  Auto-Exec  #####################################

;Actions On Exit
LilBuilder_OnExit:
  Dock_ShutDown()
  Control("Remove","All")
  Gui("Remove","All")
  Project("Remove")
  Dic("uninitialize")
  ExitApp
Return

#include %A_ScriptDir%           ;set include path

;external modules
#include ext\MMenu\MMenu.ahk
#include ext\Dic\Dic.ahk
#include ext\ToolTip\ToolTip.ahk

;code to include
#include wrapper\IE.ahk          ;wrappers for custom controls
#Include wrapper\CoHelper.ahk
#include wrapper\Structs.ahk
#include wrapper\Dock.ahk
#include wrapper\HiEdit.ahk
#include wrapper\RaGrid.ahk
#include wrapper\HLink.ahk

;necessary function for HLink control
HLink_Notify(p1, p2, p3){
   msgbox %p1%
}
                                 ;LilBuilder code
#include inc\Toolbox.ahk         ;Toolbox with icons for controls
#include inc\Gui.ahk             ;all functions related to user created guis
#include inc\Selector.ahk        ;selection
#include inc\Grid.ahk            ;modifying grid
#include inc\Messages.ahk        ;handling of messages
#include inc\Control.ahk         ;all functions related to control
#include inc\Project.ahk         ;project book keeping
#include inc\Menu.ahk            ;menu setup
#include inc\Tab.ahk             ;tab specific functions
#include inc\ToolTip.ahk         ;tooltip specific functions
#include inc\SaveLoad.ahk        ;save load projects

#include menu\Text.ahk            ;context menus for standard controls
#include menu\Edit.ahk
#include menu\ComboBox.ahk
#include menu\Button.ahk
#include menu\CheckBox.ahk
#include menu\Radio.ahk
#include menu\Picture.ahk
#include menu\ListBox.ahk
#include menu\ListView.ahk
#include menu\TreeView.ahk
#include menu\Tab.ahk
#include menu\HotKey.ahk
#include menu\GroupBox.ahk
#include menu\DateTime.ahk
#include menu\MonthCal.ahk
#include menu\Progress.ahk
#include menu\UpDown.ahk
#include menu\Slider.ahk
#include menu\StatusBar.ahk
#include menu\MultiSelection.ahk  ;context menu for multi selection of controls
#include menu\Gui.ahk             ;context menu for gui
#include menu\Toolbox.ahk         ;context menu for toolbox
#include menu\TrayMenu.ahk        ;context menu for tray

;###############################################################################
;#######        M  A  I  N       ###############################################
;###############################################################################

;main
LilBuilder(){
  LilBuilder_InitializeValues()          ;initalize some values

  LilBuilder_CreateTrayMenu()            ;create tray menu

  Selector_Create()                      ;create selector gui

  ToolBox_Create()                       ;create toolbox

  If (Project("StartWithEmptyWindow") AND !Project("RestoreLastSession"))
      Gui_Create()                       ;create an empty gui

  If Project("DockWindows")
      LilBuilder_DockWindows()           ;dock toolbox to empty gui

  Menu_Create()                          ;create Menus

  Messages_SetHandlers( true )           ;enable message handlers

  LilBuilder_InitializeHotkeys()         ;set up hotkeys
}

;###############################################################################
;###############################################################################
;###############################################################################

LilBuilder_InitializeValues(){
  Project("DockWindows", True)
  Project("StartWithEmptyWindow", True)
  Project("RestoreLastSession", False)
  ToolTip_ShowTips4Pos(False)
}

;###############################################################################

LilBuilder_DockWindows(){
  global
  Dock_HostId   := Selector_vGuiHWND()    ;??? works only for first gui
  Dock_ClientID := Toolbox_vHWND()

  Dock(0,-1,-10, 0,0,0, 0,130, 0,665)    ;dock toolbox to left, top

  Dock_Update(Dock_HostId)               ; :D

  DisableSystemMenuCommand(Dock_HostId)     ;disable window movement
  DisableSystemMenuCommand(Dock_ClientID)
}

;###############################################################################

LilBuilder_InitializeHotkeys(){
  ;hotkeys when selector is active
  Hotkey, IfWinActive, % "ahk_id " Selector_vHWND()
  Hotkey, DEL, Selector_DeleteControl
  Hotkey, Up, Selector_SetMoveIncrement
  Hotkey, Down, Selector_SetMoveIncrement
  Hotkey, Left, Selector_SetMoveIncrement
  Hotkey, Right, Selector_SetMoveIncrement
  Hotkey, LShift, Grid_Change
  Hotkey, LAlt, Grid_Change
  
  ;hotkeys when any user guis is active
  Hotkey, IfWinActive, ahk_group GroupOfUserGuis
  Hotkey, DEL, Selector_DeleteControl
  Hotkey, Up, Selector_SetMoveIncrement
  Hotkey, Down, Selector_SetMoveIncrement
  Hotkey, Left, Selector_SetMoveIncrement
  Hotkey, Right, Selector_SetMoveIncrement
  Hotkey, LShift, Grid_Change
  Hotkey, LAlt, Grid_Change

  Hotkey, IfWinActive
}

;###############################################################################

DisableSystemMenuCommand(hWnd = "", SystemCommand = "0xF010") {   ; SC_MOVE = 0xF010
  if !hWnd
    hWnd := WinExist("A")  ; Active window handle
  hSysMenu := DllCall("GetSystemMenu", "Int", hWnd, "Int", False)  ; Retrieve SystemMenu handle
  return DllCall("RemoveMenu", "Int", hSysMenu, "UInt", SystemCommand, "UInt", 0)  ; MF_BYCOMMAND = 0
} 

CheckRequiredAHKVersion(MinimumRequiredAHKVersion){    ;if it is a script check if minimum required version of AHK is used
    If not A_IsCompiled  {
        StringSplit, A, A_AHKVERSION, .
        StringSplit, B, MinimumRequiredAHKVersion, .
        Loop, %A0% {     ; create unique number for both versions, max. verion 999.1.2.3 leads to 999001002003
            AA := A%A_Index%
            BA := B%A_Index%
            Ax += AA * 1000 ** ( 4 - A_Index )
            Bx += BA * 1000 ** ( 4 - A_Index )
          }
        If ( Bx > Ax ) {
            msgbox, 16, Old AHK version,
              (LTrim
                This program requires a newer version of AHK.
                Installed version = %A_AHKVERSION%
                Required version = %MinimumRequiredAHKVersion%
    
                Please download latest version and install it.
                This Program will exit and open the webpage
                where you can download the latest AHK version.
              )
            run, http://www.autohotkey.com/download
            ExitApp
          }
      }
  }  
